/**
* teamgen.c - Utility for instructing the TCA module to
* add/delete a team from its database. The two processes communicate
* via a message queue
*
* Copyright (C) 2004-2005 Pantelis Frangoudis, Elias Efstathiou
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "teamgen.h"

//initialize message queue (we suppose that it has already been created by the tca or other similar module)
int init_mq() {
	mq_id = msgget(mq_key, 0600);
	return mq_id;
}

//sends message to the other process
//team_pub: The public key of the team to be added/removed
//action: ACTION_ADD_TEAM / ACTION_REMOVE_TEAM
//The pwnc_pubkey is serialized and copied to the buffer
int send_cmd(PWNC_PUBKEY *team_pub, int action) {
	MSG_BUF *msg_to_send;

	int mlen = 2*sizeof(unsigned short) + sizeof(unsigned int) + team_pub->datalen*sizeof(unsigned char*);
	ALLOC_AND_CLEAN(msg_to_send, sizeof(MSG_BUF) + mlen, MSG_BUF*); //allocate memory for msgbuf
	msg_to_send->mtype = action;
	memcpy(&(msg_to_send->mdata), &(team_pub->algorithm), sizeof(unsigned short));
	memcpy(&(msg_to_send->mdata) + sizeof(unsigned short), &(team_pub->bits), sizeof(unsigned short));
	memcpy(&(msg_to_send->mdata) + 2*sizeof(unsigned short), &(team_pub->datalen), sizeof(unsigned int));
	memcpy(&(msg_to_send->mdata) + 2*sizeof(unsigned short) + sizeof(unsigned int), team_pub->data, team_pub->datalen);

	//now we can send the message to the queue
	if (msgsnd(mq_id, msg_to_send, mlen, 0) == -1) {
		SAFE_FREE(msg_to_send);
		printf("send_cmd: Error sending message to queue\n");
		return -1;
	}
	SAFE_FREE(msg_to_send);
	return 0;
}

void print_usage() {
	fprintf(stderr, "\n###########################################################\n");
	fprintf(stderr, "Usage: teamgen <add/del> <path> <mq key>\n");
	fprintf(stderr, "<add/del>: Operation to be performed - \n");
	fprintf(stderr, "\tadd or delete the team with the specified public key\n");
	fprintf(stderr, "<path>: The file where a team's public key is stored\n");
	fprintf(stderr, "<mq key>: Message queue key\n");
	fprintf(stderr, "###########################################################\n\n");
}

int main(int argc, char ** argv) {
	int act = 0;
	PWNC_PUBKEY *pub_key = NULL;

	if (argc < 3) {
		print_usage();
		exit(1);
	}

	//if mqkey is specified, use it, else use defaults
	if (argc == 4) {
		mq_key = atoi(argv[3]);
	}

	if (init_mq() == -1) {
		fprintf(stderr, "Error creating message queue\n");
		exit(1);
	}

	if (!strcasecmp("add", argv[1])) {
		act = MTYPE_ADD_TEAM;
	}
	else if (!strcasecmp("del", argv[1])) {
		act = MTYPE_DEL_TEAM;
	}
	else {
		fprintf(stderr, "Invalid argument\n\n");
		print_usage();
		exit(1);
	}

	//read public key file from disk to a PWNC_PUBKEY structure
	pub_key = read_key_file(argv[2]);
	if (!pub_key) {
		fprintf(stderr, "Could not read public key file: %s\n", argv[2]);
		exit(1);
	}

	//now, construct and send appropriate message to queue
	if (send_cmd(pub_key, act) == -1) {
		fprintf(stderr, "Could not send message\n");
		SAFE_FREE_KEY(pub_key);
		exit(1);
	}

	return 0;
}
